@@ -1,5 +1,5 @@
 /**
- * Copyright (C) 2013-2019  Stefan Löffler
+ * Copyright (C) 2013-2023  Stefan Löffler
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -23,29 +23,18 @@ class PDFDocumentWidget : public PDFDocumentView
 	Q_OBJECT
 public:
   PDFDocumentWidget(QWidget * parent = nullptr, const double dpi = -1);
-  ~PDFDocumentWidget() override;
 
   bool load(const QString & filename);
 
   QWeakPointer<Backend::Document> document() const;
 
-  bool watchForDocumentChangesOnDisk() const {
-    if (_scene) return _scene->watchForDocumentChangesOnDisk();
-    else return false;
-  }
-  void setWatchForDocumentChangesOnDisk(const bool doWatch = true) { if (_scene) _scene->setWatchForDocumentChangesOnDisk(doWatch); }
-
-  QStringList backends() const;
-  QString defaultBackend() const;
-  void setDefaultBackend(const QString & backend);
-
-  // *TODO*: Possibly add some way to describe/choose/change the PDF backend used
+  bool watchForDocumentChangesOnDisk() const;
+  void setWatchForDocumentChangesOnDisk(const bool doWatch = true);
 
   void setResolution(const double dpi);
 
 protected:
   QSharedPointer<QtPDF::PDFDocumentScene> _scene;
-  QList<BackendInterface*> _backends;
   double _dpi;
 };
 
